searchNodes=[{"doc":"Bandit is an HTTP server for Plug and WebSock apps. As an HTTP server, Bandit's primary goal is to act as 'glue' between client connections managed by Thousand Island and application code defined via the Plug and/or WebSock APIs. As such there really isn't a whole lot of user-visible surface area to Bandit, and as a consequence the API documentation presented here is somewhat sparse. This is by design! Bandit is intended to 'just work' in almost all cases; the only thought users typically have to put into Bandit comes in the choice of which options (if any) they would like to change when starting a Bandit server. The sparseness of the Bandit API should not be taken as an indicator of the comprehensiveness or robustness of the project. Using Bandit With Phoenix Bandit fully supports Phoenix. Phoenix applications which use WebSockets for features such as Channels or LiveView require Phoenix 1.7 or later. Using Bandit to host your Phoenix application couldn't be simpler: Add Bandit as a dependency in your Phoenix application's mix.exs : { :bandit , &quot;~&gt; 1.0-pre&quot; } Add the following adapter: line to your endpoint configuration in config/config.exs : config :your_app , YourAppWeb.Endpoint , adapter : Bandit.PhoenixAdapter That's it! You should now see messages at startup indicating that Phoenix is using Bandit to serve your endpoint, and everything should 'just work'. Note that if you have set any exotic configuration options within your endpoint, you may need to update that configuration to work with Bandit; see the Bandit.PhoenixAdapter documentation for more information. Using Bandit With Plug Applications Using Bandit to host your own Plug is very straightforward. Assuming you have a Plug module implemented already, you can host it within Bandit by adding something similar to the following to your application's Application.start/2 function: def start ( _type , _args ) do children = [ { Bandit , plug : MyPlug } ] opts = [ strategy : :one_for_one , name : MyApp.Supervisor ] Supervisor . start_link ( children , opts ) end For details about writing Plug based applications, consult the excellent Plug documentation for plenty of examples &amp; tips to get started. Bandit supports the complete Plug API &amp; should work correctly with any Plug-based application. If you encounter errors using Bandit your Plug app, please do get in touch by filing an issue on the Bandit GitHub project (especially if the error does not occur with another HTTP server such as Cowboy). Configuration A number of options are defined when starting a server. The complete list is defined by the Bandit.options/0 type. Setting up an HTTPS Server By far the most common stumbling block encountered when setting up an HTTPS server involves configuring key and certificate data. Bandit is comparatively easy to set up in this regard, with a working example looking similar to the following: def start ( _type , _args ) do children = [ { Bandit , plug : MyPlug , scheme : :https , certfile : &quot;/absolute/path/to/cert.pem&quot; , keyfile : &quot;/absolute/path/to/key.pem&quot; ] opts = [ strategy : :one_for_one , name : MyApp.Supervisor ] Supervisor . start_link ( children , opts ) end WebSocket Support Bandit supports WebSocket implementations via the WebSock and WebSockAdapter libraries, which provide a generic abstraction for WebSockets (very similar to how Plug is a generic abstraction on top of HTTP). Bandit fully supports all aspects of these libraries. Applications should validate that the connection represents a valid WebSocket request before attempting an upgrade (Bandit will validate the connection as part of the upgrade process, but does not provide any capacity for an application to be notified if the upgrade is not successful). If an application wishes to negotiate WebSocket subprotocols or otherwise set any response headers, it should do so before upgrading.","ref":"Bandit.html","title":"Bandit","type":"module"},{"doc":"Starts a Bandit server using the provided arguments. See options/0 for specific options to pass to this function.","ref":"Bandit.html#start_link/1","title":"Bandit.start_link/1","type":"function"},{"doc":"Options to configure the deflate library used for HTTP compression","ref":"Bandit.html#t:deflate_options/0","title":"Bandit.deflate_options/0","type":"type"},{"doc":"Options to configure the HTTP/1 stack in Bandit enabled : Whether or not to serve HTTP/1 requests. Defaults to true max_request_line_length : The maximum permitted length of the request line (expressed as the number of bytes on the wire) in an HTTP/1.1 request. Defaults to 10_000 bytes max_header_length : The maximum permitted length of any single header (combined key &amp; value, expressed as the number of bytes on the wire) in an HTTP/1.1 request. Defaults to 10_000 bytes max_header_count : The maximum permitted number of headers in an HTTP/1.1 request. Defaults to 50 headers max_requests : The maximum number of requests to serve in a single HTTP/1.1 connection before closing the connection. Defaults to 0 (no limit) compress : Whether or not to attempt compression of responses via content-encoding negotiation as described in RFC9110§8.4 . Defaults to true deflate_options : A keyword list of options to set on the deflate library. A complete list can be found at deflate_options/0","ref":"Bandit.html#t:http_1_options/0","title":"Bandit.http_1_options/0","type":"type"},{"doc":"Options to configure the HTTP/2 stack in Bandit enabled : Whether or not to serve HTTP/2 requests. Defaults to true max_header_key_length : The maximum permitted length of any single header key (expressed as the number of decompressed bytes) in an HTTP/2 request. Defaults to 10_000 bytes max_header_value_length : The maximum permitted length of any single header value (expressed as the number of decompressed bytes) in an HTTP/2 request. Defaults to 10_000 bytes max_header_count : The maximum permitted number of headers in an HTTP/2 request. Defaults to 50 headers max_requests : The maximum number of requests to serve in a single HTTP/2 connection before closing the connection. Defaults to 0 (no limit) default_local_settings : Options to override the default values for local HTTP/2 settings. Values provided here will override the defaults specified in RFC9113§6.5.2 compress : Whether or not to attempt compression of responses via content-encoding negotiation as described in RFC9110§8.4 . Defaults to true deflate_options : A keyword list of options to set on the deflate library. A complete list can be found at deflate_options/0","ref":"Bandit.html#t:http_2_options/0","title":"Bandit.http_2_options/0","type":"type"},{"doc":"Possible top-level options to configure a Bandit server plug : The Plug to use to handle connections. Can be specified as MyPlug or {MyPlug, plug_opts} scheme : One of :http or :https . If :https is specified, you will also need to specify valid certfile and keyfile values (or an equivalent value within thousand_island_options.transport_options ). Defaults to :http port : The TCP port to listen on. This option is offered as a convenience and actually sets the option of the same name within thousand_island_options . If ia string value is passed, it will be parsed as an integer. Defaults to 4000 if scheme is :http , and 4040 if scheme is :https ip : The interface(s) to listen on. This option is offered as a convenience and actually sets the option of the same name within thousand_island_options.transport_options . Can be specified as: {1, 2, 3, 4} for IPv4 addresses {1, 2, 3, 4, 5, 6, 7, 8} for IPv6 addresses :loopback for local loopback (ie: 127.0.0.1 ) :any for all interfaces (ie: 0.0.0.0 ) {:local, &quot;/path/to/socket&quot;} for a Unix domain socket. If this option is used, the port option must be set to 0 keyfile : The path to a file containing the SSL key to use for this server. This option is offered as a convenience and actually sets the option of the same name within thousand_island_options.transport_options . If a relative path is used here, you will also need to set the otp_app parameter and ensure that the named file is part of your application build certfile : The path to a file containing the SSL certificate to use for this server. This option is offered as a convenience and actually sets the option of the same name within thousand_island_options.transport_options . If a relative path is used here, you will also need to set the otp_app parameter and ensure that the named file is part of your application build otp_app : Provided as a convenience when using relative paths for keyfile and certfile cipher_suite : Used to define a pre-selected set of ciphers, as described by Plug.SSL.configure/1 . Optional, can be either :strong or :compatible display_plug : The plug to use when describing the connection in logs. Useful for situations such as Phoenix code reloading where you have a 'wrapper' plug but wish to refer to the connection by the endpoint name startup_log : The log level at which Bandit should log startup info. Defaults to :info log level, can be set to false to disable it thousand_island_options : A list of options to pass to Thousand Island. Bandit sets some default values in this list based on your top-level configuration; these values will be overridden by values appearing here. A complete list can be found at ThousandIsland.options/0 http_1_options : A list of options to configure Bandit's HTTP/1 stack. A complete list can be found at http_1_options/0 http_2_options : A list of options to configure Bandit's HTTP/2 stack. A complete list can be found at http_2_options/0 websocket_options : A list of options to configure Bandit's WebSocket stack. A complete list can be found at websocket_options/0","ref":"Bandit.html#t:options/0","title":"Bandit.options/0","type":"type"},{"doc":"","ref":"Bandit.html#t:scheme/0","title":"Bandit.scheme/0","type":"type"},{"doc":"Options to configure the WebSocket stack in Bandit enabled : Whether or not to serve WebSocket upgrade requests. Defaults to true max_frame_size : The maximum size of a single WebSocket frame (expressed as a number of bytes on the wire). Defaults to 0 (no limit) validate_text_frames : Whether or not to validate text frames as being UTF-8. Strictly speaking this is required per RFC6455§5.6, however it can be an expensive operation and one that may be safely skipped in some situations. Defaults to true compress : Whether or not to allow per-message deflate compression globally. Note that upgrade requests still need to set the compress: true option in connection_opts on a per-upgrade basis for compression to be negotiated (see 'WebSocket Support' section below for details). Defaults to true","ref":"Bandit.html#t:websocket_options/0","title":"Bandit.websocket_options/0","type":"type"},{"doc":"Settings as defined in RFC9113§6.5.2","ref":"Bandit.HTTP2.Settings.html","title":"Bandit.HTTP2.Settings","type":"module"},{"doc":"A collection of settings as defined in RFC9113§6.5","ref":"Bandit.HTTP2.Settings.html#t:t/0","title":"Bandit.HTTP2.Settings.t/0","type":"type"},{"doc":"Logging conveniences for Bandit servers Allows dynamically adding and altering the log level used to trace connections within a Bandit server via the use of telemetry hooks. Should you wish to do your own logging or tracking of these events, a complete list of the telemetry events emitted by Bandit is described in the module documentation for Bandit.Telemetry . The logging included in this module is concerned specifically with protocol level events. Should you wish to log lower level transport concens, there are similar functions to these in the ThousandIsland.Logger module. Corresponding telemetry events are described in the module documentation for ThousandIsland.Telemetry .","ref":"Bandit.Logger.html","title":"Bandit.Logger","type":"module"},{"doc":"Start logging Bandit at the specified log level. Valid values for log level are :error and :info . Enabling a given log level implicitly enables all higher log levels as well.","ref":"Bandit.Logger.html#attach_logger/1","title":"Bandit.Logger.attach_logger/1","type":"function"},{"doc":"Stop logging Thousand Island at the specified log level. Disabling a given log level implicitly disables all lower log levels as well.","ref":"Bandit.Logger.html#detach_logger/1","title":"Bandit.Logger.detach_logger/1","type":"function"},{"doc":"Supported log levels","ref":"Bandit.Logger.html#t:log_level/0","title":"Bandit.Logger.log_level/0","type":"type"},{"doc":"A Bandit adapter for Phoenix. This adapter provides out-of-the-box support for all aspects of Phoenix 1.7 and later. Earlier versions of Phoenix will work with this adapter, but without support for WebSockets. To use this adapter, your project will need to include Bandit as a dependency: { :bandit , &quot;~&gt; 1.0-pre&quot; } Once Bandit is included as a dependency of your Phoenix project, add the following adapter: line to your endpoint configuration in config/config.exs : config :your_app , YourAppWeb.Endpoint , adapter : Bandit.PhoenixAdapter That's it! After restarting Phoenix you should see the startup message indicate that it is being served by Bandit, and everything should 'just work'. Note that if you have set any exotic configuration options within your endpoint, you may need to update that configuration to work with Bandit; see below for details. Endpoint configuration This adapter supports the standard Phoenix structure for endpoint configuration. Top-level keys for :http and :https are supported, and configuration values within each of those are interpreted as raw Bandit configuration as specified by Bandit.options/0 . Bandit's confguration supports all values used in a standard out-of-the-box Phoenix application, so if you haven't made any substantial changes to your endpoint configuration things should 'just work' for you. In the event that you have made advanced changes to your endpoint configuration, you may need to update this config to work with Bandit. Consult Bandit's documentation at Bandit.options/0 for details.","ref":"Bandit.PhoenixAdapter.html","title":"Bandit.PhoenixAdapter","type":"module"},{"doc":"The following telemetry spans are emitted by bandit [:bandit, :request, *] Represents Bandit handling a specific client HTTP request This span is started by the following event: [:bandit, :request, :start] Represents the start of the span This event contains the following measurements: monotonic_time : The time of this event, in :native units This event contains the following metadata: telemetry_span_context : A unique identifier for this span connection_telemetry_span_context : The span context of the Thousand Island :connection span which contains this request This span is ended by the following event: [:bandit, :request, :stop] Represents the end of the span This event contains the following measurements: monotonic_time : The time of this event, in :native units duration : The span duration, in :native units conn : The Plug.Conn representing this connection req_header_end_time : The time that header reading completed, in :native units req_body_start_time : The time that request body reading started, in :native units. req_body_end_time : The time that request body reading completed, in :native units req_line_bytes : The length of the request line, in octets. Includes all line breaks. Not included for HTTP/2 requests req_header_bytes : The length of the request headers, in octets. Includes all line breaks. Not included for HTTP/2 requests req_body_bytes : The length of the request body, in octets resp_start_time : The time that the response started, in :native units resp_end_time : The time that the response completed, in :native units. Not included for chunked responses resp_line_bytes : The length of the reponse line, in octets. Includes all line breaks. Not included for HTTP/2 requests resp_header_bytes : The length of the reponse headers, in octets. Includes all line breaks. Not included for HTTP/2 requests resp_body_bytes : The length of the reponse body, in octets. If the response is compressed, this is the size of the compressed payload as sent on the wire. Set to 0 for chunked responses resp_uncompressed_body_bytes : The length of the original, uncompressed body. Only included for responses which are compressed resp_compression_method : The method of compression, as sent in the Content-Encoding header of the response. Only included for responses which are compressed This event contains the following metadata: telemetry_span_context : A unique identifier for this span connection_telemetry_span_context : The span context of the Thousand Island :connection span which contains this request error : The error that caused the span to end, if it ended in error The following events may be emitted within this span: [:bandit, :request, :exception] The request for this span ended unexpectedly This event contains the following measurements: monotonic_time : The time of this event, in :native units This event contains the following metadata: telemetry_span_context : A unique identifier for this span connection_telemetry_span_context : The span context of the Thousand Island :connection span which contains this request kind : The kind of unexpected condition, typically :exit exception : The exception which caused this unexpected termination stacktrace : The stacktrace of the location which caused this unexpected termination [:bandit, :websocket, *] Represents Bandit handling a WebSocket connection This span is started by the following event: [:bandit, :websocket, :start] Represents the start of the span This event contains the following measurements: monotonic_time : The time of this event, in :native units compress : Details about the compression configuration for this connection This event contains the following metadata: telemetry_span_context : A unique identifier for this span origin_telemetry_span_context : The span context of the Bandit :request span from which this connection originated connection_telemetry_span_context : The span context of the Thousand Island :connection span which contains this request This span is ended by the following event: [:bandit, :websocket, :stop] Represents the end of the span This event contains the following measurements: monotonic_time : The time of this event, in :native units duration : The span duration, in :native units recv_text_frame_count : The number of text frames received recv_text_frame_bytes : The total number of bytes received in the payload of text frames recv_binary_frame_count : The number of binary frames received recv_binary_frame_bytes : The total number of bytes received in the payload of binary frames recv_ping_frame_count : The number of ping frames received recv_ping_frame_bytes : The total number of bytes received in the payload of ping frames recv_pong_frame_count : The number of pong frames received recv_pong_frame_bytes : The total number of bytes received in the payload of pong frames recv_connection_close_frame_count : The number of connection close frames received recv_connection_close_frame_bytes : The total number of bytes received in the payload of connection close frames recv_continuation_frame_count : The number of continuation frames received recv_continuation_frame_bytes : The total number of bytes received in the payload of continuation frames send_text_frame_count : The number of text frames sent send_text_frame_bytes : The total number of bytes sent in the payload of text frames send_binary_frame_count : The number of binary frames sent send_binary_frame_bytes : The total number of bytes sent in the payload of binary frames send_ping_frame_count : The number of ping frames sent send_ping_frame_bytes : The total number of bytes sent in the payload of ping frames send_pong_frame_count : The number of pong frames sent send_pong_frame_bytes : The total number of bytes sent in the payload of pong frames send_connection_close_frame_count : The number of connection close frames sent send_connection_close_frame_bytes : The total number of bytes sent in the payload of connection close frames send_continuation_frame_count : The number of continuation frames sent send_continuation_frame_bytes : The total number of bytes sent in the payload of continuation frames This event contains the following metadata: telemetry_span_context : A unique identifier for this span origin_telemetry_span_context : The span context of the Bandit :request span from which this connection originated connection_telemetry_span_context : The span context of the Thousand Island :connection span which contains this request error : The error that caused the span to end, if it ended in error","ref":"Bandit.Telemetry.html","title":"Bandit.Telemetry","type":"module"},{"doc":"See System.monotonic_time/0 .","ref":"Bandit.Telemetry.html#monotonic_time/0","title":"Bandit.Telemetry.monotonic_time/0","type":"function"},{"doc":"","ref":"Bandit.Telemetry.html#span_exception/4","title":"Bandit.Telemetry.span_exception/4","type":"function"},{"doc":"","ref":"Bandit.Telemetry.html#t:t/0","title":"Bandit.Telemetry.t/0","type":"opaque"},{"doc":"Raised by Bandit adapters if a body is attempted to be read more than once per request","ref":"Bandit.BodyAlreadyReadError.html","title":"Bandit.BodyAlreadyReadError","type":"exception"},{"doc":"","ref":"Bandit.HTTP2.Stream.StreamError.html","title":"Bandit.HTTP2.Stream.StreamError","type":"exception"}]